<?php

declare(strict_types=1);

namespace PrestaShop\Module\ConotoxiaPay\Configure;

use PrestaShop\Module\ConotoxiaPay\Definitions;
use PrestaShop\PrestaShop\Adapter\Entity\Configuration;
use PrestaShop\PrestaShop\Adapter\Entity\Language;
use PrestaShop\PrestaShop\Adapter\Entity\OrderState;
use PrestaShop\PrestaShop\Adapter\Entity\Tools;
use PrestaShopDatabaseException;
use PrestaShopException;

/**
 * Class responsible for the configurations needed during installation/uninstallation/use of the module.
 */
class Configurator
{
    /**
     * Adds Conotoxia Pay new payment status if not exist
     *
     * @return boolean
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function addOrderWaitingStatus(): bool
    {
        if (Configuration::get(Definitions::WAITING_STATUS)) {
            return true;
        }

        $newOrderState = new OrderState();
        $newOrderState->name = array();
        foreach (Language::getLanguages() as $language) {
            self::setCorrectLanguageForWaitingStatus($newOrderState, $language['iso_code'], $language['id_lang']);
        }
        $newOrderState->module_name = Definitions::MODULE_NAME;
        $newOrderState->send_email = false;
        $newOrderState->invoice = false;
        $newOrderState->unremovable = false;
        $newOrderState->color = '#4169E1';
        if (!$newOrderState->add()) {
            return false;
        }
        $stateId = $newOrderState->id;

        if ($stateId === false) {
            return false;
        }
        self::setWaitingStatus($stateId);
        self::copyStatusImageForWaitingStatus($stateId);
        return true;
    }

    /**
     * Updates waiting status translation
     *
     * @param Language $language
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function updateOrderWaitingStatusLanguage($language)
    {
        if (Configuration::get(Definitions::WAITING_STATUS)) {
            $orderState = new OrderState(self::getWaitingStatus());
            self::setCorrectLanguageForWaitingStatus($orderState, $language->iso_code, $language->id);
            $orderState->update();
        }
    }

    /**
     * Updates payment failed status translation
     *
     * @param Language $language
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function updatePaymentFailedStatusLanguage($language)
    {
        if (Configuration::get(Definitions::PAYMENT_FAILED_STATUS)) {
            $orderState = new OrderState(self::getPaymentFailedStatus());
            self::setCorrectLanguageForFailedPayment($orderState, $language->iso_code, $language->id);
            $orderState->update();
        }
    }

    /**
     * @param OrderState $orderState
     * @param $langIsoCode
     * @param $langId
     */
    private static function setCorrectLanguageForWaitingStatus($orderState, $langIsoCode, $langId)
    {
        if (Tools::strtolower($langIsoCode) === 'pl') {
            $orderState->name[$langId] = 'Oczekuje na potwierdzenie płatności z Cinkciarz Pay';
        } else {
            $orderState->name[$langId] = 'Awaiting for Conotoxia Pay confirmation';
        }
    }

    /**
     * @param OrderState $orderState
     * @param $langIsoCode
     * @param $langId
     */
    private static function setCorrectLanguageForFailedPayment($orderState, $langIsoCode, $langId)
    {
        if (Tools::strtolower($langIsoCode) === 'pl') {
            $orderState->name[$langId] = 'Płatność nie powiodła się w systemie Cinkciarz Pay';
        } else {
            $orderState->name[$langId] = 'Payment failed in Conotoxia Pay system';
        }
    }

    /**
     * Removes Conotoxia Pay waiting payment status
     *
     * @return bool
     * @throws PrestaShopException
     */
    public static function removeOrderWaitingStatus(): bool
    {
        if (Configuration::get(Definitions::WAITING_STATUS)) {
            $orderState = new OrderState(self::getWaitingStatus());
            $orderState->deleted = 1;
            $orderState->update();
        }
        return true;
    }

    /**
     * Get an id of payment waiting status
     * @return int
     */
    public static function getWaitingStatus(): int
    {
        return (int)Configuration::get(Definitions::WAITING_STATUS);
    }

    /**
     * Get an id of payment failed status
     * @return int
     */
    public static function getPaymentFailedStatus(): int
    {
        return (int)Configuration::get(Definitions::PAYMENT_FAILED_STATUS);
    }

    /**
     * Set an id of payment waiting status
     *
     * @param int $waitingStatus Id of payment waiting status
     * @return bool
     */
    public static function setWaitingStatus($waitingStatus)
    {
        return Configuration::updateValue(Definitions::WAITING_STATUS, $waitingStatus);
    }

    /**
     * Copies the image from 'bankwire' to the new destination
     * @param string $dest name of the new image file
     * @return void
     */
    private static function copyStatusImageForWaitingStatus($dest): void
    {
        $dir = _PS_ORDER_STATE_IMG_DIR_;
        copy($dir . _PS_OS_BANKWIRE_ . '.gif', $dir . $dest . '.gif');
    }

    /**
     * Copies the image from 'bankwire' to the new destination
     * @param string $dest name of the new image file
     * @return void
     */
    private static function copyStatusImageForPaymentFailedStatus($dest): void
    {
        $dir = _PS_ORDER_STATE_IMG_DIR_;
        copy($dir . _PS_OS_CANCELED_ . '.gif', $dir . $dest . '.gif');
    }

    /**
     * Adds Payment Failed in Conotoxia Pay Status
     *
     * @return boolean
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function addPaymentFailedStatus(): bool
    {

        if (Configuration::get(Definitions::PAYMENT_FAILED_STATUS)) {
            return true;
        }

        $newOrderState = new OrderState();
        $newOrderState->name = array();
        foreach (Language::getLanguages() as $language) {
            self::setCorrectLanguageForFailedPayment($newOrderState, $language['iso_code'], $language['id_lang']);
        }
        $newOrderState->module_name = Definitions::MODULE_NAME;
        $newOrderState->send_email = false;
        $newOrderState->invoice = false;
        $newOrderState->unremovable = false;
        $newOrderState->color = '#E74C3C';
        if (!$newOrderState->add()) {
            return false;
        }
        $stateId = $newOrderState->id;

        if ($stateId === false) {
            return false;
        }
        self::setPaymentFailedStatus($stateId);
        self::copyStatusImageForPaymentFailedStatus($stateId);
        return true;
    }

    /**
     * Removes Payment Failed in Conotoxia Pay Status
     *
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function removePaymentFailedStatus(): bool
    {
        if (Configuration::get(Definitions::PAYMENT_FAILED_STATUS)) {
            $orderState = new OrderState(self::getPaymentFailedStatus());
            $orderState->deleted = 1;
            $orderState->update();
        }
        return true;
    }

    /**
     * Set an id of payment failed status
     *
     * @param int $paymentFailedStatus Id of payment failed status
     * @return bool
     */
    public static function setPaymentFailedStatus($paymentFailedStatus)
    {
        return Configuration::updateValue(Definitions::PAYMENT_FAILED_STATUS, $paymentFailedStatus);
    }
}